Attribute VB_Name = "modAPI_Error"
' *********************************************
' API_ERR.BAS -- Copyright (c) Slightly Tilted Software
' By: L.J. Johnson       Date: 12-01-1996
' Comments:    Contains only ReturnApiErrString()
' *********************************************
Option Explicit

' -------------------------------------------------
' Used to get error messages directly from the
'    system instead of hard-coding them
' -------------------------------------------------
Private Const FORMAT_MESSAGE_FROM_SYSTEM = &H1000
Private Const FORMAT_MESSAGE_IGNORE_INSERTS = &H200

Private Declare Function FormatMessage Lib "kernel32" Alias _
    "FormatMessageA" _
      (ByVal dwFlags As Long, lpSource As Any, _
       ByVal dwMessageId As Long, _
       ByVal dwLanguageId As Long, _
       ByVal lpBuffer As String, _
       ByVal nSize As Long, _
       Arguments As Long) As Long

' --------------------------------------------------------
' Status Codes
' --------------------------------------------------------
Public Const INVALID_HANDLE_VALUE = -1&
Public Const ERROR_SUCCESS = 0&

' -------------------------------------------------
' API_ERR.BAS / ReturnApiErrString
'
' Passed an API error number, return an error
'   string
'
' Comments:    Takes an API error number, and returns
'              a descriptive text string of the error
'       Inputs:   xlngError is the number returned from
'                    the API error
'      Outputs:   Function returns the error string
'
' The original code appeared in Keith Pleas' article
'   in VBPJ, April 1996 (OLE Expert column).  Thanks,
'   Keith.
' -------------------------------------------------
Public Function ReturnApiErrString(ErrorCode As Long) As String
   Dim pstrBuffer    As String

   ' ----------------------------------------------
   ' Allocate the string, then get the system to
   '   tell us the error message associated with
   '   this error number
   ' ----------------------------------------------
   pstrBuffer = String(256, 0)
   FormatMessage FORMAT_MESSAGE_FROM_SYSTEM _
      Or FORMAT_MESSAGE_IGNORE_INSERTS, 0&, _
      ErrorCode, 0&, pstrBuffer, Len(pstrBuffer), 0&


   ' ----------------------------------------------
   ' Strip the last null, then the last CrLf pair if
   ' it exists
   ' ----------------------------------------------
   pstrBuffer = Left(pstrBuffer, InStr(pstrBuffer, vbNullChar) - 1)
   If Right$(pstrBuffer, 2) = Chr$(13) & Chr$(10) Then
      pstrBuffer = Mid$(pstrBuffer, 1, Len(pstrBuffer) - 2)
   End If


   ' ----------------------------------------------
   ' Set the return value
   ' ----------------------------------------------
   ReturnApiErrString = pstrBuffer

End Function
