VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "GenericService"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

' ---------------------------------------------
' Used for Event logging
' ---------------------------------------------
Private Const mconstModName = "A_Service.GenericService"

' ---------------------------------------------
' Private data
' ---------------------------------------------
Private mlngServiceStatusHandle        As Long
Private mtypServiceStatus              As SERVICE_STATUS
Private mlngServiceType                As enumServiceType
Private mstrComputerName               As String

' ---------------------------------------------
' Type of ServiceStart enumeration
' ---------------------------------------------
Public Enum enumServiceStart
   SERVICE_BOOT_START = 0
   SERVICE_SYSTEM_START = 1
   SERVICE_AUTO_START = 2
   SERVICE_DEMAND_START = 3
   SERVICE_DISABLED = 4
End Enum

' ---------------------------------------------
' Controls Accepted (Bit Mask) enumeration
' ---------------------------------------------
Public Enum enumControlsAccepted
   SERVICE_ACCEPT_STOP = &H1
   SERVICE_ACCEPT_PAUSE_CONTINUE = &H2
   SERVICE_ACCEPT_SHUTDOWN = &H4
End Enum

' ---------------------------------------------
' Service State enumeration -- for CurrentStateStr
' ---------------------------------------------
Public Enum enumCurrentState
   SERVICE_STOPPED = &H1
   SERVICE_START_PENDING = &H2
   SERVICE_STOP_PENDING = &H3
   SERVICE_RUNNING = &H4
   SERVICE_CONTINUE_PENDING = &H5
   SERVICE_PAUSE_PENDING = &H6
   SERVICE_PAUSED = &H7
End Enum

' ---------------------------------------------
' Enumeration for types of services displayed
' ---------------------------------------------
Public Enum enumServiceType
   ST_All = SERVICE_WIN32 Or SERVICE_DRIVER
   ST_Driver = SERVICE_DRIVER
   ST_Nondrivers = SERVICE_WIN32
End Enum

' ---------------------------------------------
' Control Codes enumeration
' ---------------------------------------------
Public Enum enumControlCodes
   SERVICE_CONTROL_STOP = &H1
   SERVICE_CONTROL_PAUSE = &H2
   SERVICE_CONTROL_CONTINUE = &H3
   SERVICE_CONTROL_INTERROGATE = &H4
   SERVICE_CONTROL_SHUTDOWN = &H5
   SERVICE_CONTROL_START = &H9
End Enum

Private m_ObjectData As ObjectData

' ---------------------------------------------
' PUBLIC method
' ---------------------------------------------
Public Function CheckStatus(xstrServiceName As String, _
                            xlngStateToCheck As Long) As Boolean
   Dim plngServerHwnd            As Long
   Dim plngServiceHwnd           As Long
   Dim ptypServiceStatus         As SERVICE_STATUS
   Dim pblnRtn                   As Boolean
   
   plngServerHwnd = OpenSCManager(mstrComputerName, _
                                  vbNullString, _
                                  SC_MANAGER_ALL_ACCESS)
   If plngServerHwnd = 0 Then
      ' Error
      Exit Function
   Else
      plngServiceHwnd = OpenService(plngServerHwnd, _
                                   xstrServiceName, _
                                   SERVICE_QUERY_STATUS)
      If plngServiceHwnd = 0 Then
         ' Error - Close SCM manager
         Call CloseServiceHandle(plngServerHwnd)
         Exit Function
      Else
         pblnRtn = QueryServiceStatus(plngServiceHwnd, ptypServiceStatus)
         
         If pblnRtn = True Then
            ' Success
            If ptypServiceStatus.dwCurrentState = xlngStateToCheck Then
               CheckStatus = True
            End If
         Else
            ' Failed
            MsgBox "Failed: Error " & Err.LastDllError & _
                   ", " & ReturnApiErrString(Err.LastDllError)
         End If
      End If
   End If
   
   Call CloseServiceHandle(plngServiceHwnd)
   Call CloseServiceHandle(plngServerHwnd)

End Function

' ---------------------------------------------
' PUBLIC method
' ---------------------------------------------
Public Function Control_Service(xlngControlCode As Long, _
                                xstrServiceName As String)
   Dim plngServerHwnd            As Long
   Dim plngServiceHwnd           As Long
   Dim plngAccessType            As Long
   Dim ptypServiceStatus         As SERVICE_STATUS
   Dim pblnRtn                   As Boolean
   
   ' Default to failure
   Control_Service = False
   
   Select Case xlngControlCode
      Case SERVICE_CONTROL_START
         plngAccessType = SERVICE_START
      Case SERVICE_CONTROL_STOP
         plngAccessType = SERVICE_STOP
      Case SERVICE_CONTROL_PAUSE
         plngAccessType = SERVICE_PAUSE_CONTINUE
      Case SERVICE_CONTROL_CONTINUE
         plngAccessType = SERVICE_PAUSE_CONTINUE
      Case SERVICE_CONTROL_INTERROGATE
         plngAccessType = SERVICE_INTERROGATE
      Case SERVICE_CONTROL_SHUTDOWN
         plngAccessType = SERVICE_STOP
      Case 128 To 255
         ' Custom message
         plngAccessType = SERVICE_USER_DEFINED_CONTROL
      Case Else
         ' Error
         Exit Function
   End Select
   
   plngServerHwnd = OpenSCManager(mstrComputerName, _
                                  vbNullString, _
                                  SC_MANAGER_ALL_ACCESS)
   If plngServerHwnd = 0 Then
      ' Error
      Exit Function
   Else
      plngServiceHwnd = OpenService(plngServerHwnd, _
                                   xstrServiceName, _
                                   plngAccessType)
      If plngServiceHwnd = 0 Then
         ' Error - Close SCM manager
         Call CloseServiceHandle(plngServerHwnd)
         Exit Function
      Else
         If xlngControlCode = SERVICE_CONTROL_START Then
            pblnRtn = StartService(plngServiceHwnd, 0&, 0&)
         Else
            pblnRtn = ControlService(plngServiceHwnd, _
                                     xlngControlCode, _
                                     ptypServiceStatus)
         End If
         
         If pblnRtn <> 0 Then
            ' Success
            Control_Service = True
         Else
            ' Failed
            MsgBox "Failed: Error " & Err.LastDllError & _
                   ", " & ReturnApiErrString(Err.LastDllError)
         End If
      End If
   End If
   
   Call CloseServiceHandle(plngServiceHwnd)
   Call CloseServiceHandle(plngServerHwnd)
   
End Function

' ---------------------------------------------
' PUBLIC method
' ---------------------------------------------
Public Function GetServiceStatus() As Variant
   Dim ptypEnumService        As ENUM_SERVICE_STATUS_PTR
   Dim ptypServiceConfig      As QUERY_SERVICE_CONFIG_PTR
   Dim pabytBuffer()          As Byte
   Dim pabytQuery()           As Byte
   Dim plngServicesRtn        As Long
   Dim plngBytesNeeded        As Long
   Dim plngResume             As Long
   Dim plngLoop               As Long
   Dim plngSCM_Hwnd           As Long
   Dim plngServiceHwnd        As Long
   Dim plngBufSize            As Long
   Dim plngRtn                As Long
   Dim pstrTmp                As String
   Dim pstrPath               As String
   Dim plngNumCols            As Long
   Dim plngNumRows            As Long
   Dim pstrServiceName        As String
   
   plngSCM_Hwnd = OpenSCManager(mstrComputerName, _
                                 vbNullString, _
                                 SC_MANAGER_ALL_ACCESS)
   If plngSCM_Hwnd Then
      plngLoop = EnumServicesStatus(plngSCM_Hwnd, _
         SERVICE_WIN32 Or SERVICE_DRIVER, _
         SERVICE_STATE_ALL, _
         ByVal 0&, _
         0&, _
         plngBytesNeeded, _
         plngServicesRtn, _
         plngResume)
      
      'The first call returns the
      '  size of the byte array.
      If plngLoop = 0 Then
         If Err.LastDllError = ERROR_MORE_DATA Then
            plngResume = 0
            plngServicesRtn = 0
            ReDim pabytBuffer(0 To plngBytesNeeded - 1)
            If EnumServicesStatus(plngSCM_Hwnd, _
               mlngServiceType, _
               SERVICE_STATE_ALL, _
               pabytBuffer(0), _
               plngBytesNeeded, _
               plngBytesNeeded, _
               plngServicesRtn, _
               plngResume) Then
               
               'The second call fills in
               '  the byte array.
               If plngServicesRtn Then
                  plngNumCols = 12
                  plngNumRows = plngServicesRtn
                  
                  Set m_ObjectData = New ObjectData
                  m_ObjectData.Initialize plngNumCols, plngNumRows
                  
                  ' Fill in the columns
                  m_ObjectData.AddColumnItem 1, 1, "Display Name"
                  m_ObjectData.AddColumnItem 1, 2, 0.26
               
                  m_ObjectData.AddColumnItem 2, 1, "Service Name"
                  m_ObjectData.AddColumnItem 2, 2, 0.12
               
                  m_ObjectData.AddColumnItem 3, 1, "Current"
                  m_ObjectData.AddColumnItem 3, 2, 0.07
               
                  m_ObjectData.AddColumnItem 4, 1, "Service Type"
                  m_ObjectData.AddColumnItem 4, 2, 0.12
               
                  m_ObjectData.AddColumnItem 5, 1, "Start Type"
                  m_ObjectData.AddColumnItem 5, 2, 0.09

                  m_ObjectData.AddColumnItem 6, 1, "Error Control"
                  m_ObjectData.AddColumnItem 6, 2, 0.2
               
                  m_ObjectData.AddColumnItem 7, 1, "Path"
                  m_ObjectData.AddColumnItem 7, 2, 0

                  m_ObjectData.AddColumnItem 8, 1, "Controls Accepted"
                  m_ObjectData.AddColumnItem 8, 2, 0
                     
                  m_ObjectData.AddColumnItem 9, 1, "LoadOrder Group"
                  m_ObjectData.AddColumnItem 9, 2, 0
                  
                  m_ObjectData.AddColumnItem 10, 1, "Tag ID"
                  m_ObjectData.AddColumnItem 10, 2, 0
                  
                  m_ObjectData.AddColumnItem 11, 1, "Dependencies"
                  m_ObjectData.AddColumnItem 11, 2, 0
                     
                  m_ObjectData.AddColumnItem 12, 1, "Logon Account"
                  m_ObjectData.AddColumnItem 12, 2, 0
                  
                  For plngLoop = 1 To plngServicesRtn
                     CopyMem ptypEnumService, pabytBuffer((plngLoop - 1) * Len(ptypEnumService)), Len(ptypEnumService)
                     
                     pstrServiceName = PointerToString(ptypEnumService.psServiceName)
                     
                     ' Now get additional info
                     plngServiceHwnd = OpenService(plngSCM_Hwnd, _
                                                  pstrServiceName, _
                                                  SERVICE_ALL_ACCESS)
                                       
                     ' Get the number of bytes needed
                     plngBufSize = 0
                     ReDim pabytQuery(0) As Byte
                     plngRtn = QueryServiceConfig(plngServiceHwnd, _
                                                 pabytQuery(0), _
                                                 plngBufSize, _
                                                 plngBytesNeeded)
                     
                     ' Now do it with the right size
                     plngBufSize = plngBytesNeeded
                     ReDim pabytQuery(0 To plngBytesNeeded - 1) As Byte
                     plngRtn = QueryServiceConfig(plngServiceHwnd, _
                                                 pabytQuery(0), _
                                                 plngBufSize, _
                                                 plngBytesNeeded)
                     
   
                     CopyMem ptypServiceConfig, pabytQuery(0), Len(ptypServiceConfig)
 
                     m_ObjectData.AddDataItem plngLoop, 1, PointerToString(ptypEnumService.psDisplayName)
                     m_ObjectData.AddDataItem plngLoop, 2, pstrServiceName
                     m_ObjectData.AddDataItem plngLoop, 3, ptypEnumService.ServiceStatus.dwCurrentState
                     m_ObjectData.AddDataItem plngLoop, 4, ptypEnumService.ServiceStatus.dwServiceType
                     m_ObjectData.AddDataItem plngLoop, 5, ptypServiceConfig.dwStartType
                     m_ObjectData.AddDataItem plngLoop, 6, ptypServiceConfig.dwErrorControl
                     m_ObjectData.AddDataItem plngLoop, 7, PointerToString(ptypServiceConfig.psBinaryPathName)
                     m_ObjectData.AddDataItem plngLoop, 8, ptypEnumService.ServiceStatus.dwControlsAccepted
                     
                     m_ObjectData.AddDataItem plngLoop, 9, PointerToString(ptypServiceConfig.psLoadOrderGroup)
                     m_ObjectData.AddDataItem plngLoop, 10, ptypServiceConfig.dwTagId
                     m_ObjectData.AddDataItem plngLoop, 11, PointerToString(ptypServiceConfig.psDependencies)
                     pstrTmp = Trim$(PointerToString(ptypServiceConfig.psServiceStartName))
                     If pstrTmp = "" Then
                        m_ObjectData.AddDataItem plngLoop, 12, "LocalSystem"
                     Else
                        m_ObjectData.AddDataItem plngLoop, 12, pstrTmp
                     End If
                     
                     plngRtn = CloseServiceHandle(plngServiceHwnd)
                     
                  Next plngLoop
               End If
            End If
         End If
      End If
      Call CloseServiceHandle(plngSCM_Hwnd)
   End If

   Set GetServiceStatus = m_ObjectData

End Function

' ---------------------------------------------
' PUBLIC Property Get
' Get the string for StartTypeStr when passed the
'     enumerated value for StartTypeStr
' ---------------------------------------------
Public Property Get StartTypeStr(xlngPassed As Long) As String
   Dim pstrTmp              As String
   
   Select Case xlngPassed
      Case SERVICE_BOOT_START
         pstrTmp = "Boot Start"
      Case SERVICE_SYSTEM_START
         pstrTmp = "System Start"
      Case SERVICE_AUTO_START
         pstrTmp = "Automatic"
      Case SERVICE_DEMAND_START
         pstrTmp = "Manual"
      Case SERVICE_DISABLED
         pstrTmp = "Disabled"
      Case Else
         pstrTmp = "Unknown"
   End Select

   StartTypeStr = pstrTmp
   
End Property

' ---------------------------------------------
' PUBLIC Property Set
' Get the computer name last used for getting
'     services
' ---------------------------------------------
Public Property Get ComputerName() As String
   ComputerName = mstrComputerName
End Property

' ---------------------------------------------
' PUBLIC Property Let
' Set the computer name to use for getting
'     services
' ---------------------------------------------
Public Property Let ComputerName(ByVal xstrComputerName As String)
   mstrComputerName = xstrComputerName
End Property

' ---------------------------------------------
' PUBLIC property
' Passed a State value, return a text string
'     for that value
' ---------------------------------------------
Public Property Get CurrentStateStr(xlngCurrentState As Long) As String
   
   Select Case xlngCurrentState
      Case SERVICE_RUNNING
         CurrentStateStr = "Running"
      Case SERVICE_STOPPED
         CurrentStateStr = "Stopped"
      Case SERVICE_PAUSED
         CurrentStateStr = "Paused"
      Case SERVICE_START_PENDING
         CurrentStateStr = "Start Pending"
      Case SERVICE_STOP_PENDING
         CurrentStateStr = "Stop Pending"
      Case SERVICE_CONTINUE_PENDING
         CurrentStateStr = "Continue Pending"
      Case SERVICE_PAUSE_PENDING
         CurrentStateStr = "Pause Pending"
      Case Else
         CurrentStateStr = "Unknown"
   End Select

End Property

' ---------------------------------------------
' PUBLIC Property Get
' Get the ServiceType the calls are currently
'     using
' ---------------------------------------------
Public Property Get ServiceType() As enumServiceType
   ServiceType = mlngServiceType
End Property

' ---------------------------------------------
' PUBLIC Property Let
' Set the ServiceType that the calls will use
'     on the next refresh
' ---------------------------------------------
Public Property Let ServiceType(ByVal lngServiceType As enumServiceType)
   
   Select Case lngServiceType
      Case ST_All, ST_Driver, ST_Nondrivers
         ' Do nothing
      Case Else
         lngServiceType = ST_All
   End Select
   
   mlngServiceType = lngServiceType

End Property

' ---------------------------------------------
' PRIVATE Method
' Given a pointer, return the text string
' ---------------------------------------------
Private Function PointerToString(ByVal xlngAddress As Long) As String
   Dim pstrText         As String
   Dim lngLength        As Long
   
   If xlngAddress Then
      lngLength = lstrlen(xlngAddress)
      If lngLength Then
         pstrText = Space$(lngLength)
         CopyMem ByVal pstrText, ByVal xlngAddress, lngLength
         PointerToString = pstrText
      End If
   End If

End Function

' ---------------------------------------------
' PUBLIC Property Get ServiceTypeStr
' Passed an enumerated value, return a text
'     string
' ---------------------------------------------
Public Property Get ServiceTypeStr(xlngPassed As Long) As String
   Dim pstrTmp           As String
   Dim strInteractive   As String
   
   If xlngPassed And SERVICE_INTERACTIVE_PROCESS Then
      xlngPassed = xlngPassed - SERVICE_INTERACTIVE_PROCESS
      strInteractive = "Interactive"
   Else
      strInteractive = ""
   End If
   
   Select Case xlngPassed
      Case SERVICE_WIN32_OWN_PROCESS
         If Len(strInteractive) > 0 Then
            pstrTmp = "Own Proc"
         Else
            pstrTmp = "Own Process"
         End If
      Case SERVICE_WIN32_SHARE_PROCESS
         If Len(strInteractive) > 0 Then
            pstrTmp = "Shared"
         Else
            pstrTmp = "Shared Process"
         End If
      Case SERVICE_KERNEL_DRIVER
         pstrTmp = "NT Device Drive"
      Case SERVICE_FILE_SYSTEM_DRIVER
         pstrTmp = "NT File System"
      Case Else
         pstrTmp = "Unknown"
   End Select

   If Len(strInteractive) > 0 Then
      ServiceTypeStr = strInteractive & "/" & pstrTmp
   Else
      ServiceTypeStr = pstrTmp
   End If
   
End Property

' ---------------------------------------------
' PUBLIC Property Get
' Convert an Error Control enumeration into
'     a text string
' ---------------------------------------------
Public Property Get ErrCtrlStr(xlngPassed As Long) As String
   Dim pstrTmp              As String
   
   Select Case xlngPassed
      Case SERVICE_ERROR_IGNORE
         pstrTmp = "Ignore Errors"
      Case SERVICE_ERROR_NORMAL
         pstrTmp = "Normal Error Trapping"
      Case SERVICE_ERROR_SEVERE
         pstrTmp = "Servere Error Trapping"
      Case SERVICE_ERROR_CRITICAL
         pstrTmp = "Critical Error Trapping"
      Case Else
         pstrTmp = "Unknown"
   End Select

   ErrCtrlStr = pstrTmp
   
End Property

' ---------------------------------------------
' PRIVATE Class Method
' Default to non-drivers
' ---------------------------------------------
Private Sub Class_Initialize()
   ' Initialize the service type
   mlngServiceType = ST_Nondrivers
End Sub
