Attribute VB_Name = "modServiceConstants"
' ------------------------------------------------------
' Name:     ServiceConstants.BAS
' Copyright (c) 1997 Slightly Tilted Software
' By:       L.J. Johnson       Date: 06-07-1997
' Comments: Constants and declarations for
'           making an NT Service
' ------------------------------------------------------
Option Explicit

Public Type SERVICE_STATUS
   dwServiceType              As Long
   dwCurrentState             As Long
   dwControlsAccepted         As Long
   dwWin32ExitCode            As Long
   dwServiceSpecificExitCode  As Long
   dwCheckPoint               As Long
   dwWaitHint                 As Long
End Type

Public Type ENUM_SERVICE_STATUS_PTR
   psServiceName As Long
   psDisplayName As Long
   ServiceStatus As SERVICE_STATUS
End Type

Public Type QUERY_SERVICE_LOCK_STATUS
   fIsLocked                  As Long
   lpLockOwner                As String
   dwLockDuration             As Long
End Type

Public Type QUERY_SERVICE_CONFIG
   dwServiceType              As Long
   dwStartType                As Long
   dwErrorControl             As Long
   lpBinaryPathName           As String
   lpLoadOrderGroup           As String
   dwTagId                    As Long
   lpDependencies             As String
   lpServiceStartName         As String
   lpDisplayName              As String
End Type

Public Type QUERY_SERVICE_CONFIG_PTR
   dwServiceType              As Long
   dwStartType                As Long
   dwErrorControl             As Long
   psBinaryPathName           As Long
   psLoadOrderGroup           As Long
   dwTagId                    As Long
   psDependencies             As Long
   psServiceStartName         As Long
   psDisplayName              As Long
End Type

Public Type SERVICE_TABLE_ENTRY
   lpServiceName              As String
   lpServiceProc              As Long
   lNull0                     As Long
   lNull1                     As Long
End Type

Public Const ERROR_MORE_DATA As Long = 234&

Public Const DELETE = &H10000
Public Const STANDARD_RIGHTS_READ = &H20000
Public Const STANDARD_RIGHTS_WRITE = &H20000
Public Const STANDARD_RIGHTS_EXECUTE = &H20000
Public Const STANDARD_RIGHTS_REQUIRED = &HF0000
Public Const STANDARD_RIGHTS_ALL = &H1F0000

' ---------------------------------------------
' Value to indicate no change to an optional parameter
' ---------------------------------------------
Public Const SERVICE_NO_CHANGE = &HFFFF

' ---------------------------------------------
' Service State -- for Enum Requests (Bit Mask)
' ---------------------------------------------
Public Const SERVICE_ACTIVE = &H1
Public Const SERVICE_INACTIVE = &H2
Public Const SERVICE_STATE_ALL = (SERVICE_ACTIVE Or SERVICE_INACTIVE)

' ---------------------------------------------
'
' ---------------------------------------------
'Public Const SERVICE_ACTIVE = &H1
'Public Const SERVICE_INACTIVE = &H2

' ---------------------------------------------
' Service Control Manager object specific access types
' ---------------------------------------------
Public Const SC_MANAGER_CONNECT = &H1
Public Const SC_MANAGER_CREATE_SERVICE = &H2
Public Const SC_MANAGER_ENUMERATE_SERVICE = &H4
Public Const SC_MANAGER_LOCK = &H8
Public Const SC_MANAGER_QUERY_LOCK_STATUS = &H10
Public Const SC_MANAGER_MODIFY_BOOT_CONFIG = &H20

Public Const SC_MANAGER_ALL_ACCESS = (STANDARD_RIGHTS_REQUIRED Or _
                                      SC_MANAGER_CONNECT Or _
                                      SC_MANAGER_CREATE_SERVICE Or _
                                      SC_MANAGER_ENUMERATE_SERVICE Or _
                                      SC_MANAGER_LOCK Or _
                                      SC_MANAGER_QUERY_LOCK_STATUS Or _
                                      SC_MANAGER_MODIFY_BOOT_CONFIG)

' ---------------------------------------------
' Service object specific access type
' ---------------------------------------------
Public Const SERVICE_QUERY_CONFIG = &H1
Public Const SERVICE_CHANGE_CONFIG = &H2
Public Const SERVICE_QUERY_STATUS = &H4
Public Const SERVICE_ENUMERATE_DEPENDENTS = &H8
Public Const SERVICE_START = &H10
Public Const SERVICE_STOP = &H20
Public Const SERVICE_PAUSE_CONTINUE = &H40
Public Const SERVICE_INTERROGATE = &H80
Public Const SERVICE_USER_DEFINED_CONTROL = &H100

Public Const SERVICE_ALL_ACCESS = (STANDARD_RIGHTS_REQUIRED Or _
                                   SERVICE_QUERY_CONFIG Or _
                                   SERVICE_CHANGE_CONFIG Or _
                                   SERVICE_QUERY_STATUS Or _
                                   SERVICE_ENUMERATE_DEPENDENTS Or _
                                   SERVICE_START Or _
                                   SERVICE_STOP Or _
                                   SERVICE_PAUSE_CONTINUE Or _
                                   SERVICE_INTERROGATE Or _
                                   SERVICE_USER_DEFINED_CONTROL)
Public Const SERVICE_WIN32_OWN_PROCESS = &H10
Public Const SERVICE_WIN32_SHARE_PROCESS = &H20
Public Const SERVICE_INTERACTIVE_PROCESS = &H100

Public Const SERVICE_KERNEL_DRIVER = 1
Public Const SERVICE_FILE_SYSTEM_DRIVER = 2
Public Const SERVICE_ADAPTER = 4
Public Const SERVICE_RECOGNIZER_DRIVER = 8

Public Const SERVICE_DRIVER = SERVICE_KERNEL_DRIVER Or _
                             SERVICE_FILE_SYSTEM_DRIVER Or _
                             SERVICE_RECOGNIZER_DRIVER
Public Const SERVICE_WIN32 = SERVICE_WIN32_OWN_PROCESS Or _
                             SERVICE_WIN32_SHARE_PROCESS

' ---------------------------------------------
'
' ---------------------------------------------
Public Const SERVICE_ERROR_IGNORE = 0
Public Const SERVICE_ERROR_NORMAL = 1
Public Const SERVICE_ERROR_SEVERE = 2
Public Const SERVICE_ERROR_CRITICAL = 3

' ---------------------------------------------
' API declarations, general
' ---------------------------------------------
Public Declare Function MessageBeep _
   Lib "user32" _
   (ByVal wType As Long) As Long
Public Declare Sub Sleep _
   Lib "kernel32" _
   (ByVal dwMilliseconds As Long)
Public Declare Sub CopyMem _
   Lib "kernel32" Alias "RtlMoveMemory" _
   (pTo As Any, _
    pFrom As Any, _
    ByVal lCount As Long)
Public Declare Function lstrlen _
   Lib "kernel32" Alias "lstrlenA" _
   (ByVal psString As Long) As Long

' ---------------------------------------------
' API declarations, NT Service specific
' ---------------------------------------------
Public Declare Function ChangeServiceConfig _
   Lib "advapi32.dll" Alias "ChangeServiceConfigA" _
   (ByVal hService As Long, _
    ByVal dwServiceType As Long, _
    ByVal dwStartType As Long, _
    ByVal dwErrorControl As Long, _
    ByVal lpBinaryPathName As String, _
    ByVal lpLoadOrderGroup As String, _
    lpdwTagId As Long, _
    ByVal lpDependencies As String, _
    ByVal lpServiceStartName As String, _
    ByVal lpPassword As String, _
    ByVal lpDisplayName As String) As Long
Public Declare Function CloseServiceHandle _
   Lib "advapi32.dll" _
   (ByVal hSCObject As Long) As Long
Public Declare Function ControlService _
   Lib "advapi32.dll" _
   (ByVal hService As Long, _
    ByVal dwControl As Long, _
    lpServiceStatus As SERVICE_STATUS) As Long
Public Declare Function CreateService _
   Lib "advapi32.dll" Alias "CreateServiceA" _
   (ByVal hSCManager As Long, _
    ByVal lpServiceName As String, _
    ByVal lpDisplayName As String, _
    ByVal dwDesiredAccess As Long, _
    ByVal dwServiceType As Long, _
    ByVal dwStartType As Long, _
    ByVal dwErrorControl As Long, _
    ByVal lpBinaryPathName As String, _
    ByVal lpLoadOrderGroup As String, _
    ByVal lpdwTagId As Long, _
    ByVal lpDependencies As String, _
    ByVal lp As String, _
    ByVal lpPassword As String) As Long
Public Declare Function DeleteService _
   Lib "advapi32.dll" _
   (ByVal hService As Long) As Long
Public Declare Function EnumDependentServices _
   Lib "advapi32.dll" Alias "EnumDependentServicesA" _
   (ByVal hService As Long, _
    ByVal dwServiceState As Long, _
    lpServices As ENUM_SERVICE_STATUS_PTR, _
    ByVal cbBufSize As Long, _
    pcbBytesNeeded As Long, _
    lpServicesReturned As Long) As Long
Public Declare Function EnumServicesStatus _
   Lib "advapi32.dll" Alias "EnumServicesStatusA" _
   (ByVal hSCManager As Long, _
    ByVal dwServiceType As Long, _
    ByVal dwServiceState As Long, _
    lpServices As Any, _
    ByVal cbBufSize As Long, _
    pcbBytesNeeded As Long, _
    lpServicesReturned As Long, _
    lpResumeHandle As Long) As Long
Public Declare Function GetServiceKeyName _
   Lib "advapi32.dll" Alias "GetServiceKeyNameA" _
   (ByVal hSCManager As Long, _
    ByVal lpDisplayName As String, _
    ByVal lpServiceName As String, _
    lpcchBuffer As Long) As Long
Public Declare Function GetServiceDisplayName _
   Lib "advapi32.dll" Alias "GetServiceDisplayNameA" _
   (ByVal hSCManager As Long, _
    ByVal lpServiceName As String, _
    ByVal lpDisplayName As String, _
    lpcchBuffer As Long) As Long
Public Declare Function LockServiceDatabase _
   Lib "advapi32.dll" _
   (ByVal hSCManager As Long) As Long
Public Declare Function NotifyBootConfigStatus _
   Lib "advapi32.dll" _
   (ByVal BootAcceptable As Long) As Long
Public Declare Function OpenSCManager _
   Lib "advapi32.dll" Alias "OpenSCManagerA" _
   (ByVal lpMachineName As String, _
    ByVal lpDatabaseName As String, _
    ByVal dwDesiredAccess As Long) As Long
Public Declare Function OpenService _
   Lib "advapi32.dll" Alias "OpenServiceA" _
   (ByVal hSCManager As Long, _
    ByVal lpServiceName As String, _
    ByVal dwDesiredAccess As Long) As Long
Public Declare Function QueryServiceConfig _
   Lib "advapi32.dll" Alias "QueryServiceConfigA" _
   (ByVal hService As Long, _
    lpServiceConfig As Any, _
    ByVal cbBufSize As Long, _
    pcbBytesNeeded As Long) As Long
Public Declare Function QueryServiceLockStatus _
   Lib "advapi32.dll" Alias "QueryServiceLockStatusA" _
   (ByVal hSCManager As Long, _
    lpLockStatus As QUERY_SERVICE_LOCK_STATUS, _
    ByVal cbBufSize As Long, _
    pcbBytesNeeded As Long) As Long
Public Declare Function QueryServiceObjectSecurity _
   Lib "advapi32.dll" _
   (ByVal hService As Long, _
    ByVal dwSecurityInformation As Long, _
    lpSecurityDescriptor As Any, _
    ByVal cbBufSize As Long, _
    pcbBytesNeeded As Long) As Long
Public Declare Function QueryServiceStatus _
   Lib "advapi32.dll" _
   (ByVal hService As Long, _
    lpServiceStatus As SERVICE_STATUS) As Long
Public Declare Function RegisterServiceCtrlHandler _
   Lib "advapi32.dll" Alias "RegisterServiceCtrlHandlerA" _
   (ByVal lpServiceName As String, _
    ByVal lpHandlerProc As Long) As Long
Public Declare Function SetServiceObjectSecurity _
   Lib "advapi32.dll" _
   (ByVal hService As Long, _
    ByVal dwSecurityInformation As Long, _
    lpSecurityDescriptor As Any) As Long
Public Declare Function SetServiceStatus _
   Lib "advapi32.dll" _
   (ByVal hServiceStatus As Long, _
    lpServiceStatus As SERVICE_STATUS) As Long
Public Declare Function StartServiceCtrlDispatcher _
   Lib "advapi32.dll" Alias "StartServiceCtrlDispatcherA" _
   (lpServiceStartTable As SERVICE_TABLE_ENTRY) As Long
Public Declare Function StartService _
   Lib "advapi32.dll" Alias "StartServiceA" _
   (ByVal hService As Long, _
    ByVal dwNumServiceArgs As Long, _
    ByVal lpServiceArgVectors As Long) As Long
Public Declare Function UnlockServiceDatabase _
   Lib "advapi32.dll" _
   (ScLock As Any) As Long
